# Name

IBM&reg; TNCP

# Introduction

## Summary
This CASE provides an operator to install TNC Performance 1.4 which consists of the following inventory items:
* TNCP Operator inventory item that contains the resources required to install the TNC Performance Operator through the Operator Lifecycle Manager (OLM) or via command line (CLI) application of kubernetes manifests in both an online and offline airgap environment.
* TNCP Operator inventory item that contains the custom resource and launch scripts to install/delete the custom resources

## Features

Telco Network Cloud Manager - Performance is a cloud-based application to monitor network performance. It provides a comprehensive view of the performance metrics to improve the service to your customers. You can customize what metrics you want to monitor across your network infrastructure to deliver the best quality of service.

Telco Network Cloud Manager - Performance system consists of many microservices. The microservices can be installed in a cluster of servers (a distributed system).

* Telco Network Cloud Manager - Performance architecture: 
IBM® Telco Network Cloud Manager - Performance provides a comprehensive view of the performance metrics and improve the service to your customers. You can customize what metrics you want to monitor across your network infrastructure to deliver the quality of service. With its sophisticated troubleshooting capabilities and statistical measurements, network engineers can quickly identify and fix potential quality problems before the customers can experience quality dips.
* Technology pack content: 
Telco Network Cloud Manager - Performance provides an extensive library of off-the-shelf network interfaces that are called technology packs, which can be quickly deployed, extended, and modified to manage different vendors and technologies on a single system. Technology packs are independent modules that provide technology-specific data models, vendor-specific metrics and key performance indicators (KPIs), and reports and graphs.
* Designer Tool: 
Telco Network Cloud Manager - Performance comes with Designer Tool, which is a simple browser-based tool to create highly functional business dashboards from various data sources. You can create complex, dynamic, and interactive visual representations from various data sources and display historical and real-time data. You don’t need any programming or SQL query language skills to use it.

# Details

## Prerequisites
The following prerequisites are required for a successful installation.
- [Basic prerequisites](#basic-prerequisites)
- [Configure pull Secrets to avoid docker.io rate limits](#configure-pull-secrets)
- [Configure Storage Classes](#configure-storage-classes)

### Basic prerequisites
The following prerequisites are required for a successful installation.
* Install the Red Hat OpenShift Container Platform - [4.6](https://docs.openshift.com/container-platform/4.6/welcome/index.html) or later. The installation steps are currently documented for OpenShift cluster on Fyre.
* Install the Red Hat OpenShift Container Platform [CLI](https://docs.openshift.com/container-platform/4.6/cli_reference/openshift_cli/getting-started-cli.html).
* Cluster has at least one amd64 node

### Configure pull secrets
To avoid encountering Docker.io rate limits, update the pull scretes for docker.io.

### Configure Storage Classes
- Follow the instruction mention in IBM Knowledgecenter to setup the ceph storage class [Ceph](https://www-ibm.com/support/knowledgecenter/SSUJN4_1.0/released-driver/install/prerequisites.html#set-up-storage-by-using-cephfs-storage-manager)
  - Using Ansible Playbook for installing csi-cephfs onto Fyre OCP+Beta clusters.
    ```bash
    yum install git
    cat ~/.ssh/id_rsa.pub
    
    dnf install https://dl.fedoraproject.org/pub/epel/epel-release-latest-8.noarch.rpm -y
    dnf install ansible -y
    git clone https://github.com/IBM/community-automation.git
    git clone https://github.com/rook/rook.git
    
    cd community-automation/ansible/csi-cephfs-fyre-play/
    cp examples/inventory .
    vi inventory     # Update with IP address and root password of inf node.
    cat inventory   # To ensure updates have been made.
    
    oc get sc      # To list existing SCs, if any.
    ansible-playbook  -i inventory csi-cephfs.yml
    oc get sc      # To list new SCs.
    ```
### Resources Required
Minimum scheduling capacity:
**Note**: This bare minimum setup will not provide production-like
performance; however, it will be sufficient for demonstration purposes.
- 6 virtual machines in your Kubernetes cluster, assigned as follows:
    - 1 infrustructure node: 4 CPU, 8 GB Memory 300GB Disk.
    - 1 master node: 8 CPU, 16 GB Memory 300GB Disk
    - 1 IBM foundation service Node: 4 CPU, 16 GB Memory 500GB Disk
    - 3 Worker nodes: 8 CPU, 16 GB Memory 300GB Disk
- NFS and Ceph are currently the storage types supported.

# Installing
# Installing ibm-tnc-performance

This operator can be installed in an on-line or air-gapped cluster through either of following install paths :
1. Operator Life Cycle Manager (default)
2. Kubernetes CLI

### Download the case

1. Create a directory to save the CASE to a local directory

    ```
    mkdir /tmp/cases
    cd /tmp/cases
    wget http://10.53.16.164:8080/job/mercury-v2.3.0.0/lastSuccessfulBuild/artifact/target/ibm-tncp-case.tgz
    ```
### Configure image pull secret

1. (Optional) Create a image pull secret if installing operator from authenticated registry.

    ```
    kubectl create secret docker-registry 'cp.icr.io'  \
        --docker-server='cp.icr.io'                    \
        --docker-username='cp'                         \
        --docker-password='<token>'                    \
        --docker-email='<user>'                        \
        --namespace='<target namespace>'
    ```
### Dowload cloudctl    
    
1. Download the cloudctl utility to launch the CASE Bundle
   ```bash
   sudo wget https://github.com/IBM/cloud-pak-cli/releases/download/v3.7.0/cloudctl-linux-amd64.tar.gz
   tar -xvf cloudctl-linux-amd64.tar.gz
   sudo mv cloudctl-linux-amd64 /usr/bin/cloudctl
   ```
   ```bash
   cloudctl version
   Client Version: v3.7.0+2059.cf04c19074c143e0c709f4600069e10a04b73e95
   ```
## To install on-line using Openshift Operator Catalog

1. Install the operator via OLM

    ```
    cloudctl case launch                                   \
        --case /tmp/cases/ibm-tncp-case.tgz                \
        --namespace <target namespace>                     \
        --inventory operator                               \
        --action install-catalog                           \
        --tolerance 1
    ```
2. Verify the tncp catalog pod is up
    ```
    [root@tncpcluster3-inf tncp_setup]# oc get pods -n openshift-marketplace|grep tncp
    ibm-tncp-catalog-s6lqx      1/1     Running     0          17h
    ```
3. Install tncp operator from operator hub **Operators > OperatorHub > TNCP Operator > Install**. Verify tncp operator pod started in **openshift-operators** namespace
    ```
    [root@tncpcluster3-inf tncp_setup]# oc get pods -n openshift-operators|grep tncp
    tncp-operator-6f5786bd5c-rhhtm         1/1     Running   0          75s
    ```
    
4. Select the namespace where you want to install tncp instance. Create TNCP Operator instance by navigating to **Operators > Installed Operators > TNCP Operator > Create Instance**. Provide the storage class name as per storage created in pre requisite steps.

5. Verify the tncp pods are up, wait for dashboard pod to be up
    ````
    [root@tncpcluster3-inf tncp_setup]# oc get pods -n tncp
    NAME                READY   STATUS    RESTARTS   AGE
    app-0               1/1     Running   0          13m
    cassandra-0         1/1     Running   0          13m
    dashboard-0         1/1     Running   1          13m
    diamond-db-0        1/1     Running   0          13m
    diamond-db-read-0   1/1     Running   0          13m
    inventory-0         1/1     Running   0          13m
    kafka-0             1/1     Running   0          13m
    nifi-0              1/1     Running   0          13m
    postgres-0          1/1     Running   0          13m
    postgres-th-0       1/1     Running   0          13m
    security-0          1/1     Running   0          13m
    threshold-0         1/1     Running   0          13m
    timeseries-0        1/1     Running   0          13m
    ui-0                1/1     Running   3          13m
    zookeeper-0         1/1     Running   0          14m
    ```
6. Get the TNCP Routes information to access the dashboard
    ```
    [root@tncpcluster3-inf tncp_setup]# oc get routes -n tncp
    NAME        HOST/PORT                                          PATH   SERVICES    PORT    TERMINATION            WILDCARD
    dashboard   dashboard-tncp.apps.tncpcluster3.cp.fyre.ibm.com          dashboard   https   passthrough/Redirect   None
    nifi        nifi-tncp.apps.tncpcluster3.cp.fyre.ibm.com               nifi        30026                          None
    ```
7. Launch the TNCP Dashboard UI using the route obtained in the above step.
   - URL - https://dashboard-<namespace>.apps.<cluster_name>.cp.fyre.ibm.com/
   - User (default) - npiadmin   
   - Password (default) - npiadmin
  
## To uninstall on-line using Openshift Operator Catalog

1. Uninstall the operator via OLM

    ```
    cloudctl case launch                                   \
        --case /tmp/cases/ibm-tncp-case                    \
        --namespace <target namespace>                     \
        --inventory operator                               \
        --action uninstall-catalog                         \
        --tolerance 1
    ```

## To install on-line using command-line

1. Install the operator via command line, make sure to setup ceph storage class -> [Configure Storage Classes](#configure-storage-classes)
    ```
    cloudctl case launch                                   \
        --case /tmp/cases/ibm-tncp-case.tgz                \
        --namespace <target namespace>                     \
        --inventory operator                               \
        --action installOperator                           \
        --tolerance 1
    ```


## To uninstall on-line using command-line

    ```
    cloudctl case launch                                   \
        --case /tmp/cases/ibm-tncp-case                    \
        --namespace <target namespace>                     \
        --inventory operator                               \
        --action uninstallOperator                         \
        --tolerance 1
    ```
### In Air-Gapped OpenShift Cluster With a Bastion

#### 1. Prepare Bastion Host

* Logon to the bastion machine

* Verify that the bastion machine has access
  * to public internet (to download CASE and images)
  * a target image registry ( where the images will be mirrored)
  * a target openshift cluster to install the operator

* Download and install dependent command line tools
  * [oc](https://docs.openshift.com/container-platform/3.6/cli_reference/get_started_cli.html#installing-the-cli) - To interact with Openshift Cluster
  * [cloud-pak-cli](https://github.com/IBM/cloud-pak-cli) - To download and install CASE

All the following steps should be run from the bastion machine

#### 2. Download CASE

Create a directory to save the CASE to a local directory

```bash
$ mkdir /tmp/cases
```

Run

```bash
$ cloudctl case save --case case/ibm-tncp-case --outputdir /tmp/cases
Downloading and extracting the CASE ...
- Success
Retrieving CASE version ...
- Success
Validating the CASE ...
- Success
Creating inventory ...
- Success
Finding inventory items
- Success
Resolving inventory items ...
Parsing inventory items
- Success
```

Verify the CASE and images csv has been downloaded

```bash
$ ls /tmp/cases/
total 32K
drwxr-xr-x 2 jey  64 Apr 22 23:37 charts/
-rw-r--r-- 1 jey 22K Apr 22 23:37 ibm-tncp-case-1.4.0.tgz
-rw-r--r-- 1 jey  32 Apr 22 23:37 ibm-tncp-case-1.4.0-charts.csv
-rw-r--r-- 1 jey 795 Apr 22 23:37 ibm-tncp-case-1.4.0-images.csv
```

#### 3. Configure Registry Auth

1. Create auth secret for the the source image registry

Create registry secret for source image registry (if the registry is public which doesn't require credentials, this step can be skipped)

```bash
$ cloudctl case launch                   \
    --case case/ibm-tncp-case            \
    --namespace <target namespace>       \
    --inventory operator                 \
    --action configure-creds-airgap      \
    --args "--registry cp.icr.io --user iamapikey --pass xyz"
```

2. Create auth secret for target image registry

```bash
$ cloudctl case launch                   \
    --case case/ibm-tncp-case            \
    --namespace <target namespace>       \
    --inventory operator                 \
    --action configure-creds-airgap      \
    --args "--registry $TARGET_REGISTRY --user abc --pass xyz"
```

The credentials are now saved to `~/.airgap/secrets/<registry-name>.json`

#### 4. Mirror Images

In this step image from saved CASE (images.csv) are copied to target registry in the airgap environment

```bash
$ cloudctl case launch                                     \
    --case stable/ibm-tncp-case-bundle/case/ibm-tncp-case  \
    --namespace <target namespace>                         \
    --inventory operator                                   \
    --action mirror-images                                 \
    --args "--registry $TARGET_REGISTRY --inputDir /tmp/cases"
```

#### 5. Configure Cluster for Airgap

This steps does the following

* creates a global image pull secret for the target registry (skipped if target registry is unauthenticated)
* creates a imagesourcecontentpolicy

WARNING:

* Cluster resources must adjust to the new pull secret, which can temporarily limit the usability of the cluster. Authorization credentials are stored in $HOME/.airgap/secrets and /tmp/airgap* to support this action

* Applying imagesourcecontentpolicy causes cluster nodes to recycle.

```bash
$ cloudctl case launch                  \
    --case case/ibm-tncp-case           \
    --namespace <target namespace>      \
    --inventory operator                \
    --action configure-cluster-airgap   \
    --args "--registry $TARGET_REGISTRY --inputDir /tmp/cases"
```

### 6. Install Catalog Source

This step installs the operator catalogsource in the openshift-marketplace namespace. And also installs catalogs from dependent subcases (couchdb in this case)

```bash
cloudctl case launch                    \
  --case case/ibm-tncp-case             \
  --namespace $NS                       \
  --inventory operator                  \
  --action install-catalog              \
  --args "--registry $TARGET_REGISTRY --inputDir $OFFLINEDIR --recursive"
```

### 7. Install Operator

See instructions from [Installing TNCP](#installing-ibm-tnc-assurance) section

### In Air-Gapped OpenShift Cluster Without a Bastion

#### 1. Prepare a portable device

Prepare a portable device (such as laptop) that be used to download the case and images can be carried into the air gapped environment

* Verify that the portable device has access
  * to public internet (to download CASE and images)
  * a target image registry ( where the images will be mirrored)
  * a target openshift cluster to install the operator

* Download and install dependent command line tools
  * [oc](https://docs.openshift.com/container-platform/3.6/cli_reference/get_started_cli.html#installing-the-cli) - To interact with Openshift Cluster
  * [cloud-pak-cli](https://github.com/IBM/cloud-pak-cli) - To download and install CASE

All the following steps should be run from the portable device

#### 2. Download CASE

See instructions from previous [Downloading CASE](#2-download-case) section

#### 3. Configure Registry Auth

See instructions from previous [Configure Registry Auth](#3-configure-registry-auth) section

#### 4. Mirror Images

See instructions from previous [Mirror Images](#4-mirror-images) section

#### 5. Configure Cluster for Airgap

See instructions from previous [Configure Cluster for Airgap](#5-configure-cluster-for-airgap) section

### 6. Install Catalog Source

See instructions from previous [Install Catalog Source](#6-install-catalog-source) section

### 7. Install the operator

See instructions from previous [Installing TNCP](#installing-ibm-tnc-assurance) section

## Configuration

## Storage

* Supporting all filesystem-based storage providers. Make sure that there is a valid storage provider, provisioner and storage class before installing TNCP. e.g. Rook Ceph storage system.

## Limitations

## Documentation

# PodSecurityPolicy Requirements

* Pods are using following security policy
```
securityContext:
  capabilities:
    drop:
      - ALL
  privileged: false
  runAsNonRoot: true
  readOnlyRootFilesystem: false
  allowPrivilegeEscalation: false
```
# SecurityContextConstraints Requirements

* SecurityContextConstraints will be applied automatically by operator.

Custom SecurityContextConstraints definition:

```
apiVersion: security.openshift.io/v1
kind: SecurityContextConstraints
metadata:
  annotations:
    kubernetes.io/description: "This policy is the most restrictive for {Insert Product Name},
      requiring pods to run with a non-root UID, and preventing pods from accessing the host.
      The UID and GID will be bound by ranges specified at the Namespace level."
    cloudpak.ibm.com/version: "1.1.0"
  name: tncp-scc
allowHostDirVolumePlugin: false
allowHostIPC: false
allowHostNetwork: false
allowHostPID: false
allowHostPorts: false
allowPrivilegedContainer: false
allowPrivilegeEscalation: false
allowedCapabilities: null
allowedFlexVolumes: null
allowedUnsafeSysctls: null
defaultAddCapabilities: null
defaultAllowPrivilegeEscalation: false
forbiddenSysctls:
  - "*"
fsGroup:
  type: MustRunAs
  ranges:
  - max: 65535
    min: 1
readOnlyRootFilesystem: false
requiredDropCapabilities:
- ALL
runAsUser:
  type: MustRunAsRange
seccompProfiles:
- docker/default
# This can be customized for seLinuxOptions specific to your host machine
seLinuxContext:
  type: RunAsAny
supplementalGroups:
  type: MustRunAs
  ranges:
  - max: 65535
    min: 1
# This can be customized to host specifics
volumes:
- configMap
- downwardAPI
- emptyDir
- persistentVolumeClaim
- projected
- secret
priority: 0
```
