# (C) Copyright IBM Corp. 2020  All Rights Reserved.
#
# This script implements/overrides the base functions defined in launch.sh
# This implementation is specific to this ibm-cem operator
# ibm-cem operator specific variables
caseName="ibm-tncp-case"
inventory="operator"
caseCatalogName="$caseName-catalog"
catalogNamespace="openshift-marketplace"
channelName="GA"
# - variables specific to catalog/operator installation
catalogNamespace="openshift-marketplace"
csvName="tncp-operator.v1.4.1"

install_catalog() {
    echo "-------------Installing Catalog-------------"

    $kubernetesCLI create namespace $namespace
    $kubernetesCLI config set-context --current --namespace=$namespace

    if [[ -x "$(command -v oc)"  ]]; then
      oc project $namespace
    fi

    local catalog_source_file="${casePath}/inventory/${inventory}/files/olm/catalog_source.yaml"
    $kubernetesCLI apply -f "${catalog_source_file}"

}

install_operator() {
    echo "-------------Installing Operator-------------"

    kubectl create namespace $namespace
    kubectl config set-context --current --namespace=$namespace

    if [[ -x "$(command -v oc)"  ]]; then
        oc project $namespace
    fi

    if oc get apiservices >/dev/null 2>&1; then
       local tncp_scc_file="${casePath}/inventory/${inventory}/files/cli/tncp-scc.yaml"
       sed <"${tncp_scc_file}" -e "s|\${namespace}|${namespace}|g" | sed "s|REPLACE_NAMESPACE|${namespace}|g" | tee >($kubernetesCLI apply -n "${namespace}" -f -) 1>/dev/null

       local tncp_crd_file="${casePath}/inventory/${inventory}/files/cli/tncp-crd.yaml"
       sed <"${tncp_crd_file}" -e "s|\${namespace}|${namespace}|g" | sed "s|REPLACE_NAMESPACE|${namespace}|g" | tee >($kubernetesCLI apply -n "${namespace}" -f -) 1>/dev/null
    else
       echo "Current user does not have permission to create TNC-P cluster scope resources, make sure required resources are already created using admin user"
    fi

    local operator_file="${casePath}/inventory/${inventory}/files/cli/operator.yaml"
    sed <"${operator_file}" -e "s|REPLACE_NAMESPACE|${namespace}|g" | tee >($kubernetesCLI apply -n "${namespace}" -f -) 1>/dev/null

    echo Install complete.
}


uninstall_catalog() {
    echo "-------------Uninstalling Catalog-------------"

    local catalog_source_file="${casePath}/inventory/${inventory}/files/olm/catalog_source.yaml"
    
    $kubernetesCLI delete -f "${catalog_source_file}" -n openshift-marketplace --ignore-not-found=true
    
    $kubernetesCLI delete tncp  --all > /dev/null
    
    sleep 5
    
    csv_name=`$kubernetesCLI get csv |grep tncp-operator|awk '{print $1}'`
    $kubernetesCLI delete subscription/${csv_name} -n openshift-operators --ignore-not-found=true
    $kubernetesCLI delete csv/${csv_name} -n openshift-operators --ignore-not-found=true
    
    $kubernetesCLI delete crd/tncps.tncp.ibm.com --ignore-not-found=true
    echo Uninstall complete.
}

uninstall_operator() {

    kubectl config set-context --current --namespace=$namespace

    if [[ -x "$(command -v oc)"  ]]; then
        oc project $namespace
    fi
    
    $kubernetesCLI delete tncp --all -n "${namespace}" > /dev/null
    sleep 10

    local operator_file="${casePath}/inventory/${inventory}/files/cli/operator.yaml"
    sed <"${operator_file}" -e "s|REPLACE_NAMESPACE|${namespace}|g" | tee >($kubernetesCLI delete -n "${namespace}" -f -) 1>/dev/null

    if oc get apiservices >/dev/null 2>&1; then
       local tncp_scc_file="${casePath}/inventory/${inventory}/files/cli/tncp-scc.yaml"
       sed <"${tncp_scc_file}" -e "s|\${namespace}|${namespace}|g" | sed "s|REPLACE_NAMESPACE|${namespace}|g" | tee >($kubernetesCLI delete -n "${namespace}" -f -) 1>/dev/null
    else
       echo "Current user does not have permission to delete TNC-P cluster scope resources, make sure to remove resources using admin user"
    fi

    echo Uninstall complete.
}

install() {
    install_operator
}

uninstall() {
    uninstall_operator
}
