#!/bin/bash

PRODUCT_NAME=tncp

if [[ `kubectl get namespaces | awk '{print $1}' | grep $PRODUCT_NAME | wc -l` -eq 0 ]]; then
  echo "$PRODUCT_NAME namespace not found. Exiting ..."
  exit
else
  if [[ -x "$(command -v oc)"  ]]; then
    oc project $PRODUCT_NAME >> install.log 2>&1
  else
    kubectl config set-context --current --namespace=$PRODUCT_NAME >> install.log 2>&1
  fi
fi

for parameter in "$@"
do
    if [[ "$parameter" == "--"*"="* ]]; then
        eval "${parameter/--/}"
    fi
done

if [[ -z "$storage" ]]; then
   storage='nfs'
fi
export storage

echo Uninstalling postgres...
kubectl delete statefulset postgres > /dev/null 2>&1
kubectl delete service postgres > /dev/null 2>&1
kubectl delete statefulset postgres-th > /dev/null 2>&1
kubectl delete service postgres-th > /dev/null 2>&1

echo Uninstalling cassandra...
kubectl delete statefulset cassandra > /dev/null 2>&1
kubectl delete service cassandra > /dev/null 2>&1

echo Uninstalling zookeeper...
kubectl delete statefulset zookeeper > /dev/null 2>&1
kubectl delete service zookeeper > /dev/null 2>&1

echo Uninstalling kafka...
kubectl delete statefulset kafka > /dev/null 2>&1
kubectl delete service kafka > /dev/null 2>&1

#echo Uninstalling kafka connect...
#kubectl delete statefulset kafka-connect > /dev/null 2>&1
#kubectl delete service kafka-connect > /dev/null 2>&1

#echo Uninstalling kafka schema registry...
#kubectl delete statefulset kafka-schema-registry > /dev/null 2>&1
#kubectl delete service kafka-schema-registry > /dev/null 2>&1

echo Uninstalling nifi...
kubectl delete statefulset nifi > /dev/null 2>&1
kubectl delete service nifi > /dev/null 2>&1

echo Uninstalling diamond-db...
kubectl delete statefulset diamond-db > /dev/null 2>&1
kubectl delete service diamond-db > /dev/null 2>&1
kubectl delete service diamond-db-cluster > /dev/null 2>&1
kubectl delete statefulset diamond-db-read > /dev/null 2>&1
kubectl delete service diamond-db-read > /dev/null 2>&1
kubectl delete service diamond-db-cluster-read > /dev/null 2>&1
kubectl delete statefulset diamond-db-export > /dev/null 2>&1
kubectl delete service diamond-db-export > /dev/null 2>&1
kubectl delete service diamond-db-cluster-export > /dev/null 2>&1

echo Uninstalling security service...
kubectl delete statefulset security > /dev/null 2>&1
kubectl delete service security > /dev/null 2>&1

echo Uninstalling timeseries service...
kubectl delete statefulset timeseries > /dev/null 2>&1
kubectl delete service timeseries > /dev/null 2>&1

echo Uninstalling inventory service...
kubectl delete statefulset inventory > /dev/null 2>&1
kubectl delete service inventory > /dev/null 2>&1

echo Uninstalling analytic services...
kubectl delete statefulset analytics-stream > /dev/null 2>&1
kubectl delete service analytics-stream > /dev/null 2>&1
kubectl delete statefulset analytics-batch > /dev/null 2>&1
kubectl delete service analytics-batch > /dev/null 2>&1

echo Uninstalling snmp collector...
kubectl delete statefulset snmp-collector > /dev/null 2>&1

echo Uninstalling snmp discovery...
kubectl delete statefulset snmp-discovery > /dev/null 2>&1
kubectl delete service snmp-discovery > /dev/null 2>&1

echo Uninstalling file-collector...
kubectl delete statefulset file-collector > /dev/null 2>&1
kubectl delete service file-collector > /dev/null 2>&1

echo Uninstalling ui service...
kubectl delete statefulset ui > /dev/null 2>&1
kubectl delete service ui > /dev/null 2>&1

echo Uninstalling dashboard service...
kubectl delete statefulset dashboard > /dev/null 2>&1
kubectl delete service dashboard > /dev/null 2>&1

echo Uninstalling threshold service...
kubectl delete statefulset threshold > /dev/null 2>&1
kubectl delete service threshold > /dev/null 2>&1

echo Uninstalling application service...
kubectl delete statefulset app > /dev/null 2>&1
kubectl delete service app > /dev/null 2>&1

echo Uninstalling operator...
kubectl delete statefulset operator > /dev/null 2>&1
kubectl delete service operator > /dev/null 2>&1

echo Uninstalling flow-analytics service...
kubectl delete statefulset flow-analytics > /dev/null 2>&1
kubectl delete service flow-analytics > /dev/null 2>&1

echo Uninstalling flow collector service...
kubectl delete statefulset flow-collector > /dev/null 2>&1
kubectl delete service flow-collector > /dev/null 2>&1
kubectl delete service flow-collector-external > /dev/null 2>&1

echo Uninstalling dns-collector service...
kubectl delete statefulset dns-collector > /dev/null 2>&1
kubectl delete service dns-collector > /dev/null 2>&1

echo Uninstalling ping-collector...
kubectl delete statefulset ping-collector > /dev/null 2>&1
kubectl delete service ping-collector > /dev/null 2>&1

echo Uninstalling pack-service...
kubectl delete statefulset pack-service > /dev/null 2>&1
kubectl delete service pack-service > /dev/null 2>&1

echo Uninstalling analytics-batch-latedata-service...
kubectl delete statefulset analytics-batch-latedata > /dev/null 2>&1
kubectl delete service analytics-batch-latedata > /dev/null 2>&1

echo Uninstalling config-synchronizer...
kubectl delete statefulset config-synchronizer > /dev/null 2>&1
kubectl delete service config-synchronizer > /dev/null 2>&1

echo Uninstalling data-synchronizer...
kubectl delete statefulset data-synchronizer > /dev/null 2>&1
kubectl delete service data-synchronizer > /dev/null 2>&1


if [[ $storage = "nfs" ]]; then
  echo Uninstalling nfs...
  PODCOUNT=1
  while [ $PODCOUNT -ne 0 ]
  do
    echo Waiting for $PODCOUNT pods to be terminated...
    PODCOUNT=`kubectl get pods | grep -v NAME | grep -v nfs | wc -l`  >> install.log 2>&1
    sleep 10
  done

  if [[ $1 ]]
  then
    if [ $1 == '-f' ]
    then
      echo Clean up data from NFS...
      NFS=`kubectl get pods --selector=service=nfs | grep nfs | grep Running | cut -d' ' -f1`
      kubectl exec $NFS -- bash -c "rm -rf /storage/*"
    fi
  fi
fi

if [[ $storage = "ceph" ]]; then
  echo Uninstalling ceph...
  kubectl -n rook-ceph delete storageclass ceph >> install.log 2>&1
  kubectl -n rook-ceph delete cephcluster rook-ceph >> install.log 2>&1

  CEPH_CLUSTER="1"
  while [ $CEPH_CLUSTER -ne 0 ]
  do
    CEPH_CLUSTER=`kubectl -n rook-ceph get cephcluster 2>/dev/null | grep -v rook-ceph | wc -l` >> install.log 2>&1
    sleep 10
  done

  envsubst < services/ceph/common.yaml | kubectl delete -f - >> install.log 2>&1
  envsubst < services/ceph/operator.yaml | kubectl delete -f - >> install.log 2>&1

  for CRD in $(kubectl get crd -n rook-ceph | awk '/ceph.rook.io/ {print $1}'); do
    kubectl patch crd -n rook-ceph $CRD --type merge -p '{"metadata":{"finalizers": [null]}}'
  done
fi

kubectl delete rc nfs > /dev/null 2>&1
kubectl delete service nfs > /dev/null 2>&1

kubectl delete configmap common > /dev/null 2>&1

kubectl delete ServiceAccount pod-access tncp > /dev/null 2>&1
kubectl delete Role pod-access tncp-operator > /dev/null 2>&1
kubectl delete RoleBinding pod-access tncp-operator > /dev/null 2>&1
kubectl delete ClusterRole pod-access tncp-operator > /dev/null 2>&1
kubectl delete ClusterRoleBinding pod-access tncp-operator > /dev/null 2>&1

kubectl delete deployment,statefulset,service,rc,pvc,tncp --all > /dev/null 2>&1

kubectl get pv | awk -v prodname="^$PRODUCT_NAME/" '$6 ~ prodname { print $1 }' | xargs -i{} kubectl delete pv {}

kubectl delete namespace $PRODUCT_NAME 2>&1

echo Uninstall complete.
